<?php

/******************************************************************************
  Indexer script.  It will only work if called from the root docmgr directory
******************************************************************************/

/******************************************************************************
    preliminary configuration and variable setting
******************************************************************************/

//make sure this is being run from the command line.  We just die here
//to not give anyone any clues as to what this file is
if (php_sapi_name()!="cli") die;

//define our alternate file path to find everything important.  I hope to get rid of this
//define("ALT_FILE_PATH","../../");

//get our includes
//first call the config file to get our settings, call our base functions, and get our wrapper
include("config/config.php");
include("include/postgresql.inc.php");
include("include/function.inc.php");
include("include/module.inc.php");
include("include/array_function.inc.php");
include("include/email_function.inc.php");
include("app/common.inc.php");
include("app/object.inc.php");
include("include/index_function.inc.php");

//include xml parser.  Use simplexml if available
if (function_exists("simplexml_load_string")) include("include/simplexml.inc.php");
else include("include/legacyxml.inc.php");

//set our path defines
define("DATA_DIR",FILE_DIR."/data");
define("TMP_DIR",FILE_DIR."/tmp");

//set to use short tags
ini_set("short_open_tag","1");

//load our center modules to see the objects
$siteModArr = loadSiteStructure("modules/center/");
$_SESSION["siteModList"] = $siteModArr["list"];
$_SESSION["siteModInfo"] = $siteModArr["info"];
$siteModList = &$_SESSION["siteModList"];
$siteModInfo = &$_SESSION["siteModInfo"];
        
//load our objects so we can call their indexing functions
loadObjects();

//setup which apps are available to docmgr
setExternalApps();

//connect to the db
$conn = db_connect(DBHOST,DBUSER,DBPASSWORD,DBPORT,DBNAME);

//allow indexing of a certain objectId.  This is for debugging only
if (defined("DEV_MODE") && in_array("--index-object",$argv)) {

  //we are looking for the id passed after our parameter
  $key = array_search("--index-object",$argv) + 1;
  $obj = $argv[$key];
  
  if ($obj) {

    //index the item    
    echo "Indexing object ".$obj."\n";
    runObjectIndex($conn,$obj);

  } else echo "Invalid object id passed\n";
  
  die;
  
}


/*******************************************************************************
    Now it's time to process our batch.  We continue to run until
    there are no more batches left
*******************************************************************************/


while (1) {

    //get the ids of all the objects in this batch
    $sql = "SELECT * FROM dm_index_queue ORDER BY id";
    $list = total_result($conn,$sql);
    $objArr = &$list["object_id"];

    debugMsg(1,"No objects found in the queue.  Exiting");

    //get out if there's nothing to do
    if (!$list["count"]) exit;

    debugMsg(1,$list["count"]." objects found in the queue.  Proceeding to index");

    //if this is a tsearch2 configuration, we need to go ahead and index the name and summaries
    //for all files.  This should be quick, and will make the files be returned if a user 
    //tries a search before the query fails
    if (defined("TSEARCH2_INDEX"))  foreach ($objArr AS $obj) runObjectIndex($conn,$obj,1);
    
    for ($i=0;$i<$list["count"];$i++) {
      
      $obj = $list["object_id"][$i];
      $id = $list["id"][$i];

      //index the item    
      runObjectIndex($conn,$obj);

      //delete this item from the queue
      $sql = "DELETE FROM dm_index_queue WHERE id='$id'";
      db_query($conn,$sql);

    }

    //sleep 10 seconds before checking for another batch
    sleep(5);

}


function runObjectIndex($conn,$objId,$propOnly = null) {

  //first, get our object's name and summary
  $sql = "SELECT id,name,object_type FROM dm_object WHERE id='$objId'";
  $objInfo = single_result($conn,$sql);

  $name = &$objInfo["name"];
  $summary = &$objInfo["summary"];
  $objectType = $objInfo["object_type"];

  //hand off to the object's specific function.  This should return any content in the
  //object to be indexed below.

  debugMsg(1,"Now indexing ".$name);

  //start creating our index string if using tsearch2
  if (defined("TSEARCH2_INDEX")) {

    //remove the file extension from the name before we index
    $pos = strrpos($name,".");
    $noext = substr($name,0,$pos);
 
    //throw these into any array and spit out a unique version of the file name.Since we
    //are indexing the name twice, technically, this prevents the same word from being added
    //twice and falsely increasing the weight
    $idxName = uniqueString($name." ".$noext);
 
    //setup our indexing string with weighting. 'A' for the filename, 'B' for
    //the summary, and the default 'D' for the content
    $indexString = "setweight(to_tsvector('".TSEARCH2_PROFILE."','".sanitize($idxName)."'),'A')||
                      setweight(to_tsvector('".TSEARCH2_PROFILE."','".sanitize($summary)."'),'B') ";

    //if propOnly is passed, index the objetc's properties and exit
    //we'll only handle tsearch2 here.
    if ($propOnly) {

      $sql = "DELETE FROM dm_index WHERE object_id='$objId';";
      if ($indexString) $sql .= "INSERT INTO dm_index (object_id,idxfti) VALUES ('$objId',$indexString);";
      if (db_query($conn,$sql)) return true;
      else return false;

    }

  }

  //run the objects function that returns its content, if there is any
  $className = $objectType."Object";

  debugMsg(3,"Trying to load runIndex function in object class ".$className);

  $c = loadClassMethod($className,"runIndex");
  if (is_object($c)) {
    debugMsg(3,"Function found, extracting content");
    $content = $c->runIndex($conn,$objId);
  }
  else {
    debugMsg(3,"runIndex function not found in ".$className);
    $content = null;
  }
  
  //shorten it to the word limit if set
  if (defined("INDEX_WORD_LIMIT") && $content) {
    $arr = explode(" ",$content);
    $arr = array_slice($arr,0,INDEX_WORD_LIMIT);
    $content = implode(" ",$arr);
  }

  //prepare the content for indexing
  if ($content) $content = string_clean($content);

  debugMsg(5,"\n----BEGIN INDEXED CONTENT----\n ".$content." \n----END INDEXED CONTENT----\n");

  //finish off our tsearch2 string and insert it.  We leave content untouched so the tsearch2 indexer can
  //index and rank it appropriately
  if (defined("TSEARCH2_INDEX")) {
  
    if ($content) $indexString .= " || setweight(to_tsvector('".TSEARCH2_PROFILE."','".sanitize($content)."'),'D') ";

    $sql = "DELETE FROM dm_index WHERE object_id='$objId';";
    $sql .= "INSERT INTO dm_index (object_id,idxfti) VALUES ('$objId',$indexString);";
    if (db_query($conn,$sql)) return true;
    else return false;

  //store content for the old indexer.  We need to clean up the content before we store it in our index
  } else {
  
    $sql = "DELETE FROM dm_index WHERE object_id='$objId';";
    if ($content) $sql .= "INSERT INTO dm_index (object_id,idxtext) VALUES ('$objId','".sanitize($content)."');";
    if (db_query($conn,$sql)) return true;
    else return false;
  
  }


}

