<?

function loadSiteStructure($dir) {

	$arr = findModConfig($dir);
	$num = count($arr);

	$cacheFile = $dir."/module-cache.xml";
	
	//if a cached version of the structure exists, get it instead
	//of search through all of the directories
	if (file_exists($cacheFile)) {
		$str = file_get_contents($cacheFile);
		return return_modlist($str);
	}		

	$str = outputXmlHeader();

	for ($row=0;$row<$num;$row++) {

		$path = $arr[$row];

		//dynamically add the path of the module.  This should be faster overall
		$tmp = file_get_contents($arr[$row]);

		$path = str_replace("module.xml","",$path);

		$search = "<module>\n";

		$replace = "<module>\n\t<module_path>".$path."</module_path>\n";
		$replace .= "\t<owner_path>".getOwnerPath($path)."</owner_path>\n";
		$str .= str_replace($search,$replace,$tmp);

	}

	$str .= outputXmlFooter();

	return return_modlist($str);

}

function setPermDefines() {

	if (!$_SESSION["definePermArray"]) {

                if (defined("ALT_FILE_PATH")) $xmlFile = ALT_FILE_PATH."/config/permissions.xml";
                else $xmlFile = "config/permissions.xml";
	
		$str = file_get_contents("$xmlFile");
		$_SESSION["definePermArray"] = return_permlist($str);

	}

	$permArray = $_SESSION["definePermArray"];

	if ($permArray) {

		for ($row=0;$row<count($permArray["name"]);$row++) {

			$dn = $permArray["define_name"][$row];
			$bitVal = bitCal($permArray["bitpos"][$row]);
			define("$dn","$bitVal");

		}


	} else return false;

}

function setCustomPermDefines() {

	if (!$_SESSION["defineCustomPermArray"]) {

                if (defined("ALT_FILE_PATH")) $xmlFile = ALT_FILE_PATH."/config/customperm.xml";
                else $xmlFile = "config/customperm.xml";

		if (file_exists("$xmlFile")) {
	
			$str = file_get_contents("$xmlFile");
			$_SESSION["defineCustomPermArray"] = return_permlist($str);

		} else $_SESSION["defineCustomPermArray"] = "ignore";

	}

	//skip if there are no perms to worry about
	if ($_SESSION["defineCustomPermArray"]=="ignore") return false;

	$permArray = $_SESSION["defineCustomPermArray"];

	if ($permArray) {

		for ($row=0;$row<count($permArray["name"]);$row++) {

			$dn = $permArray["define_name"][$row];
			$bitVal = bitCal($permArray["bitpos"][$row]);
			define("$dn","$bitVal");
			
		}

	} else return false;

}

function outputXmlHeader() {

    return "<data>\n";
      
}
      
function outputXmlFooter() {
      
	return "</data>\n\n";

}

function findModConfig($directory,$resultArray=null) {

	$handle=@opendir($directory);

	while ($file = @readdir($handle)) {

		//skip directory markers
		if ($file!="." && $file!=".." && $file!=".svn") {

			//skip directories if asked
			if (is_dir($directory.$file)) $resultArray = findModConfig($directory.$file."/",$resultArray);
			elseif ($file=="module.xml") $resultArray[] = $directory.$file;

		}

	}
	@closedir($handle);

	return $resultArray;

}

function return_modlist($data) {

	$info = array();
	$list = array();
	
	$multi = array("permissions","custom_perm");

	// To call the function
	$tree = returnXMLTree($data,"parseGenericXMLTree",$multi);

	$info = array();

	if (!is_array($tree)) return false;
	
	$num = count($tree["module_name"]);
	$fields = array_keys($tree);
	
	$fieldNum = count($fields);

	//check for duplicates
	for ($row=0;$row<$num;$row++) {
	
		$key = $tree["link_name"][$row];

		//if this value has already been set, then there are duplicate modules 
		//I.E., two modules with the same link name
		if (is_array($info[$key])) 
			die("Module Error!  Two modules have the link name <b>\"".$info[$key]["link_name"]."\"</b>.");

		//transpose our array.  This allows for easy module access later		
		for ($i=0;$i<$fieldNum;$i++) {
		
			$field = $fields[$i];
			if (isset($tree[$field][$row])) $info[$key][$field] = $tree[$field][$row];	

		}

	}

	
	//sort by sort order by default
	$tree = arrayMultiSort($tree,"sort_order");

	$arr["list"] = $tree;
	$arr["info"] = $info;

	return $arr;	

}

function return_permlist($data) {

	$list = array();
	
	// To call the function
	$tree = returnXMLTree($data,"parseGenericXMLTree");

	return $tree;

}



/* Function will attempt to open the xmlloc as a local file, on fail it will attempt to open it as a web link */
function returnXMLTree($data,$func,$multi = null) { 

	if (!$func) return false;

	$parser = xml_parser_create('ISO-8859-1');
	xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 1); 
	xml_parse_into_struct($parser, $data, $vals, $index); 
	xml_parser_free($parser); 

	return $func($vals,$multi);

} 


function parseGenericXMLTree($vals,$multi = null) {

	$num = count($vals);
	$i = 0;

	for ($row=0;$row<$num;$row++) {

		if (isset($vals[$row]["level"])) $level = $vals[$row]["level"];
		if (isset($vals[$row]["type"])) $type = $vals[$row]["type"];
		if (isset($vals[$row]["value"])) $value = $vals[$row]["value"];
		if (isset($vals[$row]["tag"])) $tag = strtolower($vals[$row]["tag"]);

		if ($level<2) continue;

		if (isset($vals[$row]["value"])) {
			if (@in_array($tag,$multi)) $arr[$tag][$i][] = $vals[$row]["value"];
			else $arr[$tag][$i] = $vals[$row]["value"];
		}

		if ($type=="close") $i++;

	}

	return $arr;

}

/*********************************************************
        parseGenericXML
	parses a generic xml string into an associative
	array.  
	Any tag which may contain multiple
	entries within a record must be specified
	in the multi() parameter
*********************************************************/
                                
function parseGenericXML($obj,$data,$multi = null) {

	// To call the function
	return returnXMLTree($data,"parseGenericXMLTree",$multi);

}




