<?php

class fileThumbnail {

	
	function createImageThumb($filepath,$thumb) {
	
		$montage = null;
		$arr = @getImageSize($filepath);
	
		if (!$arr[1]) return;
	
		$ratio = floatValue($arr[0] / $arr[1],"2");
	
		$newwidth = intValue($ratio * 100);
	
		if ($newwidth>150 || ($arr[0]<100 && $arr[1]<100)) $montage = 1;
		else $newsize = $newwidth."x100";
	
		//do not alter the image if it's too small, or shrink if it's too long
		if ($montage) `montage -size 100x100 -gravity center "$filepath" "$thumb"`;
		else `convert -size "$newsize" -thumbnail "$newsize" "$filepath" "$thumb"`;
	
	}
	
	function createTiffThumb($filepath,$thumb) {
	
		//find out if multipage, use tiffinfo
		$numpages=`tiffinfo "$filepath"`;
		$numpages=substr_count($numpages,"TIFF Directory");
	
		//only one page, just resize it and be on our way
		if ($numpages=="1") {
		
			`convert -resize 100x100 "$filepath" "$thumb"`;
		
		} else {
	
			$dir = TMP_DIR."/".rand();
			$tifprefix = $dir."/tiff";
	
			mkdir($dir);
			`tiffsplit "$filepath" $tifprefix`;
			
			$fileArr = listDirectory($dir,null,null);
	
			$file = $dir."/".$fileArr[0];	
	
			`convert -resize 100x100 "$file" "$thumb"`;
					
			//todo: replace this with php function	
			if (is_dir($dir)) `rm -r $dir`;
	
		}
	
	}
	
	function createPdfThumb($filepath,$thumb) {
	
		//create a ppm of the first page with xpdf
		if (defined("XPDF_SUPPORT")) {
	
			$prefix = TMP_DIR."/".USER_ID;
			$firstPage = $prefix."-000001.ppm";
			
			`pdftoppm -f 1 -l 1 "$filepath" $prefix`;
	
		//extract the first page of the pdf w/ ghostscript
		} else {
	
			$firstPage = TMP_DIR."/".USER_ID."-".rand().".pdf";
		
			`gs -sDEVICE=pdfwrite -dNOPAUSE -dQUIET -dBATCH -dFirstPage=1 -dLastPage=1 -sOutputFile=$firstPage "$filepath"`;
			
		}
	
		`convert -resize 100x100 "$firstPage" "$thumb"`;
	
		@unlink($firstPage);
					                    
	}

	function createOpenDocumentThumb($filepath,$thumb) {

		$temp = `unzip -p "$filepath" "Thumbnails/thumbnail.png"`;
		$fp = fopen($thumb,"wb");
		fwrite($fp,$temp);
		fclose($fp);

	       	`mogrify -size 100x100 -resize 100x100 "$thumb"`;

	}

	function createOpenOfficeThumb($filepath,$thumb) {
	
		fileThumbnail::createOpenDocumentThumb($filepath,$thumb);
		
	}                                                                                	

	
	function createTxtThumb($filepath,$thumb,$mime) {
	
		$tempfile = TMP_DIR."/".USER_ID."-".rand().".ps";
	
		if ($mime=="text/plain") {

			if (!defined("ENSCRIPT_SUPPORT")) return false;
	
			`enscript --pages=1 --word-wrap --output=$tempfile "$filepath"`;
	
	        	$str = `identify -verbose "$tempfile" | grep Geometry`;
	        	$str = eregi_replace("[^0-9x]","",$str);
	        	$arr = explode("x",$str);

	        	if (!$arr[1]) return;
	
	        	$newwidth = intValue(($arr[0] / $arr[1]) * 100);
	        	$newsize = $newwidth."x100";

	        	`convert -size $newsize -resize $newsize "$tempfile" "$thumb"`;
	
			@unlink($tempfile);
			
		}
	
	}
	
	function createDocThumb($filepath,$thumb) {
	
		//setup our temp ps file and our temp directory for extraction
		$tempfile = TMP_DIR."/".USER_ID."-".rand().".ps";
	
		if (!defined("DOC_SUPPORT")) return false;
	
		`antiword -p letter -m 8859-1.txt "$filepath" > "$tempfile"`;
	
		//make sure the file was created properly
		if (!is_file($tempfile)) return false;
		if (filesize($tempfile)==0) return false;
	
		//shrink the ps document down to one page
		$str = file_get_contents($tempfile);
		$pos = strpos($str,"%%Page: 2");
	
		//there's more than one page, shrink it down to one
		if ($pos!==FALSE) {
	
			$str = substr($str,0,$pos);
			$str .= "%%Trailer\n%%Pages: 1\n%%EOF\n";
		
			$fp = fopen($tempfile,"w");
			fwrite($fp,$str);
			fclose($fp);
		
		}
	
	       	`convert -size 77x100 -resize 77x100 "$tempfile" "$thumb"`;
		@unlink($tempfile);
	
	}
	
}